@@ -23,6 +23,7 @@
 #include <QDesktopServices>
 #include <QStandardPaths>
 
+#include "iconutils.h"
 #include "riffimportdlg.h"
 #include "ui_riffimportdlg.h"
 
@@ -40,8 +41,10 @@ RiffImportDlg::RiffImportDlg(QWidget *parent)
     connect(m_riff, &Riff::signalDLS, this, &RiffImportDlg::slotCompleted);
 
     ui->setupUi(this);
-    connect(ui->m_inputBtn, SIGNAL(clicked()), SLOT(openInput()));
-    connect(ui->m_outputBtn, SIGNAL(clicked()), SLOT(openOutput()));
+    ui->m_inputBtn->setIcon(IconUtils::GetIcon("wrench"));
+    ui->m_outputBtn->setIcon(IconUtils::GetIcon("wrench"));
+    connect(ui->m_inputBtn, &QToolButton::clicked, this, &RiffImportDlg::openInput);
+    connect(ui->m_outputBtn, &QToolButton::clicked, this, &RiffImportDlg::openOutput);
 }
 
 RiffImportDlg::~RiffImportDlg()
@@ -70,7 +73,7 @@ void RiffImportDlg::openOutput()
     dlg.setFileMode(QFileDialog::AnyFile);
     dlg.setAcceptMode(QFileDialog::AcceptSave);
     if (dlg.exec() == QFileDialog::Accepted) {
-        const QString& fileName = dlg.selectedFiles().constFirst();
+        const QString fileName = dlg.selectedFiles().constFirst();
         setOutput(fileName);
     }
 }
