@@ -1,6 +1,6 @@
 /*
     MIDI Virtual Piano Keyboard
-    Copyright (C) 2008-2022, Pedro Lopez-Cabanillas <plcl@users.sf.net>
+    Copyright (C) 2008-2023, Pedro Lopez-Cabanillas <plcl@users.sf.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -49,6 +49,7 @@ public:
 
 protected slots:
     void slotInstrument(int bank, int pc, QString name);
+    void slotPercussion(int bank, int pc, QString name);
     void slotCompleted(QString name, QString version, QString copyright);
     void openInput();
     void openOutput();
@@ -57,6 +58,7 @@ private:
     Ui::RiffImportDlg *ui;
     Riff* m_riff;
     QMap<int, Bank> m_ins;
+    QMap<int, Bank> m_perc;
     QString m_input;
     QString m_output;
     QString m_name;
