@@ -117,37 +117,50 @@ VPiano::VPiano( QWidget * parent, Qt::WindowFlags flags )
     connect(orientationGroup, &QActionGroup::triggered, this, &VPiano::slotNameOrientation);
 
     connect(ui.pianokeybd, &PianoKeybd::signalName, this, &VPiano::slotNoteName);
-    connect(ui.actionAbout, SIGNAL(triggered()), SLOT(slotAbout()));
-    connect(ui.actionAboutQt, SIGNAL(triggered()), SLOT(slotAboutQt()));
-    connect(ui.actionAboutTranslation, SIGNAL(triggered()), SLOT(slotAboutTranslation()));
-    connect(ui.actionConnections, SIGNAL(triggered()), SLOT(slotConnections()));
-    connect(ui.actionPreferences, SIGNAL(triggered()), SLOT(slotPreferences()));
-    connect(ui.actionEditKM, SIGNAL(triggered()), SLOT(slotEditKeyboardMap()));
-    connect(ui.actionContents, SIGNAL(triggered()), SLOT(slotHelpContents()));
-    connect(ui.actionWebSite, SIGNAL(triggered()), SLOT(slotOpenWebSite()));
-    connect(ui.actionImportSoundFont, SIGNAL(triggered()), SLOT(slotImportSF()));
-    connect(ui.actionEditExtraControls, SIGNAL(triggered()), SLOT(slotEditExtraControls()));
-    connect(ui.actionShortcuts, SIGNAL(triggered()), SLOT(slotShortcuts()));
-    connect(ui.actionKeyboardInput, SIGNAL(toggled(bool)), SLOT(slotKeyboardInput(bool)));
-    connect(ui.actionMouseInput, SIGNAL(toggled(bool)), SLOT(slotMouseInput(bool)));
-    connect(ui.actionTouchScreenInput, SIGNAL(toggled(bool)), SLOT(slotTouchScreenInput(bool)));
-    connect(ui.actionColorPalette, SIGNAL(triggered()), SLOT(slotColorPolicy()));
-    connect(ui.actionColorScale, SIGNAL(toggled(bool)), SLOT(slotColorScale(bool)));
-    connect(ui.actionWindowFrame, SIGNAL(toggled(bool)), SLOT(toggleWindowFrame(bool)));
-    connect(ui.actionOctave_Subscript_Designation, SIGNAL(toggled(bool)), SLOT(slotOctaveSubscript(bool)));
+    connect(ui.actionAbout, &QAction::triggered, this, &VPiano::slotAbout);
+    connect(ui.actionAboutQt, &QAction::triggered, this, &VPiano::slotAboutQt);
+    connect(ui.actionAboutTranslation, &QAction::triggered, this, &VPiano::slotAboutTranslation);
+    connect(ui.actionConnections, &QAction::triggered, this, &VPiano::slotConnections);
+    connect(ui.actionPreferences, &QAction::triggered, this, &VPiano::slotPreferences);
+    connect(ui.actionEditKM, &QAction::triggered, this, &VPiano::slotEditKeyboardMap);
+    connect(ui.actionContents, &QAction::triggered, this, &VPiano::slotHelpContents);
+    connect(ui.actionWebSite, &QAction::triggered, this, &VPiano::slotOpenWebSite);
+    connect(ui.actionImportSoundFont, &QAction::triggered, this, &VPiano::slotImportSF);
+    connect(ui.actionEditExtraControls, &QAction::triggered, this, &VPiano::slotEditExtraControls);
+    connect(ui.actionShortcuts, &QAction::triggered, this, &VPiano::slotShortcuts);
+    connect(ui.actionKeyboardInput, &QAction::toggled, this, &VPiano::slotKeyboardInput);
+    connect(ui.actionMouseInput, &QAction::toggled, this, &VPiano::slotMouseInput);
+    connect(ui.actionTouchScreenInput, &QAction::toggled, this, &VPiano::slotTouchScreenInput);
+    connect(ui.actionColorPalette, &QAction::triggered, this, &VPiano::slotColorPolicy);
+    connect(ui.actionColorScale, &QAction::toggled, this, &VPiano::slotColorScale);
+    connect(ui.actionWindowFrame, &QAction::toggled, this, &VPiano::toggleWindowFrame);
+    connect(ui.actionOctave_Subscript_Designation,
+            &QAction::toggled,
+            this,
+            &VPiano::slotOctaveSubscript);
     connect(ui.actionLoad_Configuration, &QAction::triggered, this, &VPiano::slotLoadConfiguration);
     connect(ui.actionSave_Configuration, &QAction::triggered, this, &VPiano::slotSaveConfiguration);
     // Toolbars actions: toggle view
-    connect(ui.toolBarNotes->toggleViewAction(), SIGNAL(toggled(bool)),
-            ui.actionNotes, SLOT(setChecked(bool)));
-    connect(ui.toolBarControllers->toggleViewAction(), SIGNAL(toggled(bool)),
-            ui.actionControllers, SLOT(setChecked(bool)));
-    connect(ui.toolBarBender->toggleViewAction(), SIGNAL(toggled(bool)),
-            ui.actionBender, SLOT(setChecked(bool)));
-    connect(ui.toolBarPrograms->toggleViewAction(), SIGNAL(toggled(bool)),
-            ui.actionPrograms, SLOT(setChecked(bool)));
-    connect(ui.toolBarExtra->toggleViewAction(), SIGNAL(toggled(bool)),
-            ui.actionExtraControls, SLOT(setChecked(bool)));
+    connect(ui.toolBarNotes->toggleViewAction(),
+            &QAction::toggled,
+            ui.actionNotes,
+            &QAction::setChecked);
+    connect(ui.toolBarControllers->toggleViewAction(),
+            &QAction::toggled,
+            ui.actionControllers,
+            &QAction::setChecked);
+    connect(ui.toolBarBender->toggleViewAction(),
+            &QAction::toggled,
+            ui.actionBender,
+            &QAction::setChecked);
+    connect(ui.toolBarPrograms->toggleViewAction(),
+            &QAction::toggled,
+            ui.actionPrograms,
+            &QAction::setChecked);
+    connect(ui.toolBarExtra->toggleViewAction(),
+            &QAction::toggled,
+            ui.actionExtraControls,
+            &QAction::setChecked);
 #if defined(SMALL_SCREEN)
     ui.toolBarControllers->hide();
     ui.toolBarBender->hide();
@@ -325,30 +338,27 @@ void VPiano::initToolBars()
     m_Velocity->setToolTip(QString::number(VPianoSettings::instance()->velocity()));
     m_Velocity->setFocusPolicy(Qt::NoFocus);
     ui.toolBarNotes->addWidget(m_Velocity);
-    connect( m_sboxChannel, SIGNAL(valueChanged(int)),
-             SLOT(slotChannelValueChanged(int)));
-    connect( m_sboxOctave, SIGNAL(valueChanged(int)),
-             SLOT(slotBaseOctaveValueChanged(int)) );
-    connect( m_sboxTranspose, SIGNAL(valueChanged(int)),
-             SLOT(slotTransposeValueChanged(int)) );
-    connect( m_Velocity, SIGNAL(valueChanged(int)),
-             SLOT(slotVelocityValueChanged(int)) );
-    connect( ui.actionChannelUp, SIGNAL(triggered()),
-             m_sboxChannel, SLOT(stepUp()) );
-    connect( ui.actionChannelDown, SIGNAL(triggered()),
-             m_sboxChannel, SLOT(stepDown()) );
-    connect( ui.actionOctaveUp, SIGNAL(triggered()),
-             m_sboxOctave, SLOT(stepUp()) );
-    connect( ui.actionOctaveDown, SIGNAL(triggered()),
-             m_sboxOctave, SLOT(stepDown()) );
-    connect( ui.actionTransposeUp, SIGNAL(triggered()),
-             m_sboxTranspose, SLOT(stepUp()) );
-    connect( ui.actionTransposeDown, SIGNAL(triggered()),
-             m_sboxTranspose, SLOT(stepDown()) );
-    connect( ui.actionVelocityUp, SIGNAL(triggered()),
-             SLOT(slotVelocityUp()) );
-    connect( ui.actionVelocityDown, SIGNAL(triggered()),
-             SLOT(slotVelocityDown()) );
+    connect(m_sboxChannel,
+            QOverload<int>::of(&QSpinBox::valueChanged),
+            this,
+            &VPiano::slotChannelValueChanged);
+    connect(m_sboxOctave,
+            QOverload<int>::of(&QSpinBox::valueChanged),
+            this,
+            &VPiano::slotBaseOctaveValueChanged);
+    connect(m_sboxTranspose,
+            QOverload<int>::of(&QSpinBox::valueChanged),
+            this,
+            &VPiano::slotTransposeValueChanged);
+    connect(m_Velocity, &QAbstractSlider::valueChanged, this, &VPiano::slotVelocityValueChanged);
+    connect(ui.actionChannelUp, &QAction::triggered, m_sboxChannel, &QSpinBox::stepUp);
+    connect(ui.actionChannelDown, &QAction::triggered, m_sboxChannel, &QSpinBox::stepDown);
+    connect(ui.actionOctaveUp, &QAction::triggered, m_sboxOctave, &QSpinBox::stepUp);
+    connect(ui.actionOctaveDown, &QAction::triggered, m_sboxOctave, &QSpinBox::stepDown);
+    connect(ui.actionTransposeUp, &QAction::triggered, m_sboxTranspose, &QSpinBox::stepUp);
+    connect(ui.actionTransposeDown, &QAction::triggered, m_sboxTranspose, &QSpinBox::stepDown);
+    connect(ui.actionVelocityUp, &QAction::triggered, this, &VPiano::slotVelocityUp);
+    connect(ui.actionVelocityDown, &QAction::triggered, this, &VPiano::slotVelocityDown);
     // Controllers tool bar
     m_lblControl = new QLabel(this);
     ui.toolBarControllers->addWidget(m_lblControl);
@@ -370,10 +380,11 @@ void VPiano::initToolBars()
     m_Control->setToolTip("0");
     m_Control->setFocusPolicy(Qt::NoFocus);
     ui.toolBarControllers->addWidget(m_Control);
-    connect( m_comboControl, SIGNAL(currentIndexChanged(int)),
-             SLOT(slotComboControlCurrentIndexChanged(int)) );
-    connect( m_Control, SIGNAL(sliderMoved(int)),
-             SLOT(slotControlSliderMoved(int)) );
+    connect(m_comboControl,
+            QOverload<int>::of(&QComboBox::currentIndexChanged),
+            this,
+            &VPiano::slotComboControlCurrentIndexChanged);
+    connect(m_Control, &QAbstractSlider::valueChanged, this, &VPiano::slotControlValueChanged);
     // Pitch bender tool bar
     m_lblBender = new QLabel(this);
     ui.toolBarBender->addWidget(m_lblBender);
@@ -387,10 +398,8 @@ void VPiano::initToolBars()
     m_bender->setToolTip("0");
     m_bender->setFocusPolicy(Qt::NoFocus);
     ui.toolBarBender->addWidget(m_bender);
-    connect( m_bender, SIGNAL(sliderMoved(int)),
-             SLOT(slotBenderSliderMoved(int)) );
-    connect( m_bender, SIGNAL(sliderReleased()),
-             SLOT(slotBenderSliderReleased()) );
+    connect(m_bender, &QAbstractSlider::sliderMoved, this, &VPiano::slotBenderSliderMoved);
+    connect(m_bender, &QAbstractSlider::sliderReleased, this, &VPiano::slotBenderSliderReleased);
     // Programs tool bar
     m_lblBank = new QLabel(this);
     ui.toolBarPrograms->addWidget(m_lblBank);
@@ -410,45 +419,36 @@ void VPiano::initToolBars()
     m_comboProg->setSizeAdjustPolicy(QComboBox::AdjustToContents);
     m_comboProg->setFocusPolicy(Qt::NoFocus);
     ui.toolBarPrograms->addWidget(m_comboProg);
-    connect( m_comboBank, SIGNAL(activated(int)),
-             SLOT(slotComboBankActivated(int)) );
-    connect( m_comboProg, SIGNAL(activated(int)),
-             SLOT(slotComboProgActivated(int)) );
+    connect(m_comboBank,
+            QOverload<int>::of(&QComboBox::activated),
+            this,
+            &VPiano::slotComboBankActivated);
+    connect(m_comboProg,
+            QOverload<int>::of(&QComboBox::activated),
+            this,
+            &VPiano::slotComboProgActivated);
     // Toolbars actions: buttons
-    connect( ui.actionPanic, SIGNAL(triggered()),
-             SLOT(slotPanic()));
-    connect( ui.actionResetAll, SIGNAL(triggered()),
-             SLOT(slotResetAllControllers()));
-    connect( ui.actionReset, SIGNAL(triggered()),
-             SLOT(slotResetBender()));
-    connect( ui.actionEditExtra, SIGNAL(triggered()),
-             SLOT(slotEditExtraControls()));
+    connect(ui.actionPanic, &QAction::triggered, this, &VPiano::slotPanic);
+    connect(ui.actionResetAll, &QAction::triggered, this, &VPiano::slotResetAllControllers);
+    connect(ui.actionReset, &QAction::triggered, this, &VPiano::slotResetBender);
+    connect(ui.actionEditExtra, &QAction::triggered, this, &VPiano::slotEditExtraControls);
     // Tools actions
-    connect( ui.actionNextBank, SIGNAL(triggered()),
-             SLOT(slotBankNext()) );
-    connect( ui.actionPreviousBank, SIGNAL(triggered()),
-             SLOT(slotBankPrev()) );
-    connect( ui.actionNextProgram, SIGNAL(triggered()),
-             SLOT(slotProgramNext()) );
-    connect( ui.actionPreviousProgram, SIGNAL(triggered()),
-             SLOT(slotProgramPrev()) );
-    connect( ui.actionNextController, SIGNAL(triggered()),
-             SLOT(slotControllerNext()) );
-    connect( ui.actionPreviousController, SIGNAL(triggered()),
-             SLOT(slotControllerPrev()) );
-    connect( ui.actionControllerDown, SIGNAL(triggered()),
-             SLOT(slotControllerDown()) );
-    connect( ui.actionControllerUp, SIGNAL(triggered()),
-             SLOT(slotControllerUp()) );
-    /* connect( ui.actionEditPrograms, SIGNAL(triggered()),
-             SLOT(slotEditPrograms())); */
+    connect(ui.actionNextBank, &QAction::triggered, this, &VPiano::slotBankNext);
+    connect(ui.actionPreviousBank, &QAction::triggered, this, &VPiano::slotBankPrev);
+    connect(ui.actionNextProgram, &QAction::triggered, this, &VPiano::slotProgramNext);
+    connect(ui.actionPreviousProgram, &QAction::triggered, this, &VPiano::slotProgramPrev);
+    connect(ui.actionNextController, &QAction::triggered, this, &VPiano::slotControllerNext);
+    connect(ui.actionPreviousController, &QAction::triggered, this, &VPiano::slotControllerPrev);
+    connect(ui.actionControllerDown, &QAction::triggered, this, &VPiano::slotControllerDown);
+    connect(ui.actionControllerUp, &QAction::triggered, this, &VPiano::slotControllerUp);
+    /* connect( ui.actionEditPrograms, &QAction::triggered, this, &VPiano::slotEditPrograms); */
     retranslateToolbars();
 }
 
-//void VPiano::slotDebugDestroyed(QObject *obj)
-//{
-//    qDebug() << Q_FUNC_INFO << obj->metaObject()->className();
-//}
+/*void VPiano::slotDebugDestroyed(QObject *obj)
+{
+    qDebug() << Q_FUNC_INFO << obj->metaObject()->className();
+}*/
 
 void VPiano::clearExtraControllers()
 {
@@ -496,21 +496,28 @@ void VPiano::initExtraControllers()
         ExtraControl::decodeString( s, lbl, control, type,
                                     minValue, maxValue, defValue,
                                     size, fileName, keySequence );
+        for (int ch = 0; ch < MIDICHANNELS; ++ch) {
+            if (!m_ctlState[ch].contains(control)) {
+                m_ctlState[ch][control] = defValue;
+            }
+        }
         if (m_ctlState[channel].contains(control))
             value = m_ctlState[channel][control];
         else
             value = defValue;
-        switch(type) {
+        switch (type) {
         case ExtraControl::ControlType::SwitchControl:
             chkbox = new QCheckBox(this);
             chkbox->setStyleSheet(QSTR_CHKBOXSTYLE);
             chkbox->setProperty(MIDICTLONVALUE, maxValue);
             chkbox->setProperty(MIDICTLOFFVALUE, minValue);
             chkbox->setChecked(bool(value));
-            connect(chkbox, SIGNAL(clicked(bool)), SLOT(slotControlClicked(bool)));
+            chkbox->setAutoRepeat(false);
+            connect(chkbox, &QCheckBox::clicked, this, &VPiano::slotControlClicked);
             if (!keySequence.isEmpty()) {
-                QShortcut *s = new QShortcut(QKeySequence(keySequence), chkbox, SLOT(click()));
+                QShortcut *s = new QShortcut(keySequence, this);
                 s->setAutoRepeat(false);
+                connect(s, &QShortcut::activated, chkbox, [=] { chkbox->animateClick(); });
             }
             w = chkbox;
             break;
@@ -521,7 +528,7 @@ void VPiano::initExtraControllers()
             knob->setMaximum(maxValue);
             knob->setValue(value);
             knob->setToolTip(QString::number(value));
-            connect(knob, SIGNAL(sliderMoved(int)), SLOT(slotExtraController(int)));
+            connect(knob, &QAbstractSlider::valueChanged, this, &VPiano::slotExtraController);
             w = knob;
             break;
         case ExtraControl::ControlType::SpinBoxControl:
@@ -529,7 +536,10 @@ void VPiano::initExtraControllers()
             spin->setMinimum(minValue);
             spin->setMaximum(maxValue);
             spin->setValue(value);
-            connect(spin, SIGNAL(valueChanged(int)), SLOT(slotExtraController(int)));
+            connect(spin,
+                    QOverload<int>::of(&QSpinBox::valueChanged),
+                    this,
+                    &VPiano::slotExtraController);
             w = spin;
             break;
         case ExtraControl::ControlType::SliderControl:
@@ -540,7 +550,7 @@ void VPiano::initExtraControllers()
             slider->setMaximum(maxValue);
             slider->setToolTip(QString::number(value));
             slider->setValue(value);
-            connect(slider, SIGNAL(sliderMoved(int)), SLOT(slotExtraController(int)));
+            connect(slider, &QAbstractSlider::valueChanged, this, &VPiano::slotExtraController);
             w = slider;
             break;
         case ExtraControl::ControlType::ButtonCtlControl:
@@ -548,10 +558,12 @@ void VPiano::initExtraControllers()
             button->setText(lbl);
             button->setProperty(MIDICTLONVALUE, maxValue);
             button->setProperty(MIDICTLOFFVALUE, minValue);
-            connect(button, SIGNAL(clicked(bool)), SLOT(slotControlClicked(bool)));
+            button->setAutoRepeat(false);
+            connect(button, &QToolButton::clicked, this, &VPiano::slotControlClicked);
             if (!keySequence.isEmpty()) {
-                QShortcut *s = new QShortcut(QKeySequence(keySequence), button, SLOT(animateClick()));
+                QShortcut *s = new QShortcut(keySequence, this);
                 s->setAutoRepeat(false);
+                connect(s, &QShortcut::activated, button, [=] { button->animateClick(); });
             }
             w = button;
             break;
@@ -561,10 +573,12 @@ void VPiano::initExtraControllers()
             button->setText(lbl);
             button->setProperty(SYSEXFILENAME, fileName);
             button->setProperty(SYSEXFILEDATA, readSysexDataFile(fileName));
-            connect(button, SIGNAL(clicked(bool)), SLOT(slotControlClicked(bool)));
+            button->setAutoRepeat(false);
+            connect(button, &QToolButton::clicked, this, &VPiano::slotControlClicked);
             if (!keySequence.isEmpty()) {
-                QShortcut *s = new QShortcut(QKeySequence(keySequence), button, SLOT(animateClick()));
+                QShortcut *s = new QShortcut(keySequence, this);
                 s->setAutoRepeat(false);
+                connect(s, &QShortcut::activated, button, [=] { button->animateClick(); });
             }
             w = button;
             break;
@@ -576,12 +590,13 @@ void VPiano::initExtraControllers()
                 QLabel *qlbl = new QLabel(lbl, this);
                 qlbl->setMargin(TOOLBARLABELMARGIN);
                 ui.toolBarExtra->addWidget(qlbl);
-                //connect(qlbl, SIGNAL(destroyed(QObject*)), SLOT(slotDebugDestroyed(QObject*)));
+                //connect(qlbl, &QObject::destroyed, this, &VPiano::slotDebugDestroyed);
             }
             w->setProperty(MIDICTLNUMBER, control);
+            w->setProperty(MIDICTLDEFVALUE, defValue);
             w->setFocusPolicy(Qt::NoFocus);
             ui.toolBarExtra->addWidget(w);
-            //connect(w, SIGNAL(destroyed(QObject*)), SLOT(slotDebugDestroyed(QObject*)));
+            //connect(w, &QObject::destroyed, this, &VPiano::slotDebugDestroyed);
         }
     }
 }
@@ -719,7 +734,10 @@ void VPiano::readMidiControllerSettings()
     }
     settings->endGroup();
     if (m_extraControls.isEmpty()) {
-        m_extraControls = QStringList{"Sustain,64,0,0,64,0,", "Sostenuto,66,0,0,64,0,", "Soft,67,0,0,64,0,"};
+        QString softPedal = tr("Soft") + ",67,0,0,64,0,";
+        QString sostenutoPedal = tr("Sostenuto") + ",66,0,0,64,0,";
+        QString sustainPedal = tr("Sustain") + ",64,0,0,64,0,";
+        m_extraControls << softPedal << sostenutoPedal << sustainPedal;
     }
 }
 
@@ -781,6 +799,10 @@ void VPiano::writeSettings()
         }
     }
     settings->endGroup();
+
+    writeDriverSettings(settings.getQSettings(), m_midiin);
+    writeDriverSettings(settings.getQSettings(), m_midiout);
+
     settings->sync();
 }
 
@@ -925,17 +947,15 @@ void VPiano::slotPitchBend(const int chan, const int value)
 
 void VPiano::showEvent ( QShowEvent *event )
 {
-    static bool firstTime{true};
     //qDebug() << Q_FUNC_INFO << firstTime;
-    if (firstTime) {
+    std::call_once(m_firstTime, [=] {
         if (!restoreGeometry(VPianoSettings::instance()->geometry())) {
             qWarning() << "restoreGeometry() failed!";
         }
         if (!restoreState(VPianoSettings::instance()->state())) {
             qWarning() << "restoreState() failed!";
         }
-        firstTime = false;
-    }
+    });
     QMainWindow::showEvent(event);
 #if !defined(SMALL_SCREEN)
     if (m_initialized) {
@@ -994,17 +1014,17 @@ void VPiano::sendController(const int controller, const int value)
 
 void VPiano::resetAllControllers()
 {
-    sendController(CTL_RESET_ALL_CTL, 0);
     initializeAllControllers();
+    sendController(CTL_RESET_ALL_CTL, 0);
 }
 
 void VPiano::initializeAllControllers()
 {
     int channel = VPianoSettings::instance()->channel();
+    initControllers(channel);
     int index = m_comboControl->currentIndex();
     int ctl = m_comboControl->itemData(index).toInt();
     int val = m_ctlState[channel][ctl];
-    initControllers(channel);
     m_comboControl->setCurrentIndex(index);
     m_Control->setValue(val);
     m_Control->setToolTip(QString::number(val));
@@ -1015,7 +1035,12 @@ void VPiano::initializeAllControllers()
         if (c.isValid()) {
             ctl = c.toInt();
             if (m_ctlState[channel].contains(ctl)) {
-                val = m_ctlState[channel][ctl];
+                auto v = w->property(MIDICTLDEFVALUE);
+                if (v.isValid()) {
+                    val = v.toInt();
+                } else {
+                    val = m_ctlState[channel][ctl];
+                }
                 QVariant p = w->property("value");
                 if (p.isValid()) {
                     w->setProperty("value", val);
@@ -1145,7 +1170,7 @@ void VPiano::slotExtraController(const int value)
     }
 }
 
-void VPiano::slotControlSliderMoved(const int value)
+void VPiano::slotControlValueChanged(const int value)
 {
     int index = m_comboControl->currentIndex();
     int controller = m_comboControl->itemData(index).toInt();
@@ -1198,6 +1223,20 @@ void VPiano::connectMidiInSignals()
     }
 }
 
+void VPiano::writeDriverSettings(QSettings *settings, QObject *driver)
+{
+    if (settings != nullptr && driver != nullptr) {
+        const QMetaObject *mo = driver->metaObject();
+        //const auto signature = QMetaObject::normalizedSignature("writeSettings(QSettings*)");
+        const auto index = mo->indexOfMethod("writeSettings(QSettings*)");
+        if (index != -1) {
+            /*auto res =*/
+            QMetaObject::invokeMethod(driver, "writeSettings", Q_ARG(QSettings *, settings));
+            //qDebug() << Q_FUNC_INFO << mo->className() << res;
+        }
+    }
+}
+
 void VPiano::slotConnections()
 {
     QPointer<MidiSetup> dlgMidiSetup = new MidiSetup(this);
@@ -1226,10 +1265,8 @@ void VPiano::initControllers(int channel)
 {
     if (m_ins != nullptr) {
         InstrumentData controls = m_ins->control();
-        InstrumentData::ConstIterator it, end;
-        it = controls.constBegin();
-        end = controls.constEnd();
-        for( ; it != end; ++it ) {
+        InstrumentData::ConstIterator it;
+        for (it = controls.constBegin(); it != controls.constEnd(); ++it) {
             int ctl = it.key();
             switch (ctl) {
             case CTL_VOLUME:
@@ -1371,9 +1408,7 @@ void VPiano::applyInitialSettings()
         i = m_ctlSettings[ch].begin();
         end = m_ctlSettings[ch].end();
         for (; i != end; ++i) {
-            j = m_ctlState[ch].find(i.key());
-            if (j != m_ctlState[ch].end())
-                m_ctlState[ch][i.key()] = i.value();
+            m_ctlState[ch][i.key()] = i.value();
         }
     }
     channel = VPianoSettings::instance()->channel();
@@ -1579,6 +1614,18 @@ void VPiano::updateExtraController(int ctl, int val)
     }
 }
 
+void VPiano::removeExtraController(int ctl)
+{
+    auto controls = m_ins->control();
+    if (!controls.contains(ctl)) {
+        for (int ch = 0; ch < MIDICHANNELS; ++ch) {
+            if (m_ctlState[ch].contains(ctl)) {
+                m_ctlState[ch].remove(ctl);
+            }
+        }
+    }
+}
+
 void VPiano::updateBankChange(int bank)
 {
     int idx;
@@ -1657,7 +1704,7 @@ public:
         buttonBox->setStandardButtons(QDialogButtonBox::Ok);
         layout->addWidget(buttonBox);
         browser->setSource(document);
-        connect(buttonBox, SIGNAL(accepted()), SLOT(close()));
+        connect(buttonBox, &QDialogButtonBox::accepted, this, &QDialog::close);
     }
 };
 #endif
@@ -1712,6 +1759,7 @@ void VPiano::slotImportSF()
 void VPiano::slotEditExtraControls()
 {
     QPointer<DialogExtraControls> dlgExtra = new DialogExtraControls(this);
+    connect(dlgExtra, &DialogExtraControls::controlRemoved, this, &VPiano::removeExtraController);
     dlgExtra->setControls(m_extraControls);
     releaseKb();
     if (dlgExtra->exec() == QDialog::Accepted) {
@@ -1962,13 +2010,11 @@ void VPiano::slotVelocityDown()
 void VPiano::slotControllerUp()
 {
     m_Control->triggerAction(QDial::SliderPageStepAdd);
-    slotControlSliderMoved(m_Control->value());
 }
 
 void VPiano::slotControllerDown()
 {
     m_Control->triggerAction(QDial::SliderPageStepSub);
-    slotControlSliderMoved(m_Control->value());
 }
 
 void VPiano::slotSwitchLanguage(QAction *action)
@@ -2131,17 +2177,16 @@ void VPiano::enforceMIDIChannelState()
 {
     if (VPianoSettings::instance()->enforceChannelState()) {
         int channel = VPianoSettings::instance()->channel();
-        //qDebug() << Q_FUNC_INFO << "channel=" << m_channel << endl;
         QMap<int,int>::Iterator i, end;
-        i = m_ctlSettings[channel].begin();
-        end = m_ctlSettings[channel].end();
+        i = m_ctlState[channel].begin();
+        end = m_ctlState[channel].end();
         for (; i != end; ++i) {
-            //qDebug() << "ctl=" << i.key() << "val=" << i.value();
             sendController(i.key(), i.value());
         }
-        //qDebug() << "bank=" << m_lastBank[m_channel];
+        if (m_lastBank[channel] < 0) {
+            m_lastBank[channel] = 0;
+        }
         sendBankChange(m_lastBank[channel]);
-        //qDebug() << "prog=" << m_lastProg[m_channel];
         sendProgramChange(m_lastProg[channel]);
     }
 }
@@ -2268,7 +2313,7 @@ void VPiano::slotLoadConfiguration()
 void VPiano::slotSaveConfiguration()
 {
     releaseKb();
-    auto configDir = QStandardPaths::writableLocation(QStandardPaths::AppConfigLocation);
+    const auto configDir = QStandardPaths::writableLocation(QStandardPaths::AppConfigLocation);
     QFileDialog dlg(this);
     dlg.setNameFilter(tr("Configuration files (*.conf *.ini)"));
     dlg.setDirectory(configDir);
@@ -2277,9 +2322,12 @@ void VPiano::slotSaveConfiguration()
     dlg.setFileMode(QFileDialog::AnyFile);
     dlg.setAcceptMode(QFileDialog::AcceptSave);
     if (dlg.exec() == QDialog::Accepted) {
-        auto selected = dlg.selectedFiles();
+        const auto selected = dlg.selectedFiles();
         if (!selected.isEmpty()) {
-            VPianoSettings::instance()->SaveToFile(selected.first());
+            const auto lastFileName = SettingsFactory::fileName();
+            SettingsFactory::setFileName(selected.constFirst());
+            writeSettings();
+            SettingsFactory::setFileName(lastFileName);
         }
     }
     grabKb();
