@@ -22,6 +22,8 @@
 #include <QMainWindow>
 #include <QPointer>
 #include <drumstick/pianokeybd.h>
+#include <mutex>
+
 #if defined(ENABLE_NATIVE_FILTER)
 	#include "nativefilter.h"
 #endif
@@ -122,7 +124,7 @@ protected Q_SLOTS:
     void slotNameOrientation(QAction* action);
     void slotNameVisibility(QAction* action);
     void slotNameVariant(QAction* action);
-    void slotControlSliderMoved(const int value);
+    void slotControlValueChanged(const int value);
     void slotExtraController(const int value);
     void slotControlClicked(const bool value);
     void slotBenderSliderMoved(const int pos);
@@ -209,6 +211,7 @@ private:
     void sendSysex(const QByteArray& data);
     void updateController(int ctl, int val);
     void updateExtraController(int ctl, int val);
+    void removeExtraController(int ctl);
     void updateBankChange(int bank = -1);
     void updateProgramChange(int program = -1);
     void grabKb();
@@ -269,7 +272,10 @@ private:
 #if defined(Q_OS_WINDOWS)
     WinSnap m_snapper;
 #endif
+    std::once_flag m_firstTime;
+
     void connectMidiInSignals();
+    void writeDriverSettings(QSettings *settings, QObject *driver);
 };
 
 #endif // VPIANO_H
